#ifndef  MODULES_RENDER_H
#define  MODULES_RENDER_H

/*
**   render.h
**
**   (C) Copyright 2003-2004 Rocklyte Systems.
**       All Rights Reserved.
*/

#ifndef  MAIN_H
#include <pandora/main.h>
#endif

#ifndef  MODULES_BITMAP_H
#include <pandora/modules/bitmap.h>
#endif

#define MODVERSION_RENDER ((FLOAT)1.0)

struct RenderList {
   OBJECTID ParentID;  /* Object that owns the render area */
   OBJECTID ObjectID;  /* ID of the render area */
   OBJECTID BitmapID;  /* Shared bitmap buffer, if available */
   OBJECTID FocusID;   /* Focus route (equivalent to the RouteFocus field) */
   OBJECTID ScreenID;  /* Screen */
   OBJECTID DataMID;   /* [applies to the bitmap owner] */
   LONG Flags;         /* Render flags (RNF_VISIBLE etc) */
   LONG XCoord;        /* Horizontal coordianate */
   LONG YCoord;        /* Vertical coordinate */
   LONG Width;         /* Width */
   LONG Height;        /* Height */
   /* Note: The order of the following fields must match 'struct ClipRectangle' */
   union {
      LONG AbsX;          /* Absolute X */
      LONG Left;
   };
   union {
      LONG AbsY;          /* Absolute Y */
      LONG Top;
   };
   LONG Right;         /* Absolute right coordinate */
   LONG Bottom;        /* Absolute bottom coordinate */
   WORD Level;         /* Level number within the hierarchy */
   WORD Opacity;       /* Current opacity setting 0 - 255 */
   WORD TopLimit;      /* Top movement limit */
   WORD LeftLimit;     /* Left movement limit */
   WORD RightLimit;    /* Right movement limit */
   WORD BottomLimit;   /* Bottom movement limit */
   LONG LineWidth;     /* [applies to the bitmap owner] */
   BYTE BytesPerPixel; /* [applies to the bitmap owner] */
   BYTE BitsPerPixel;  /* [applies to the bitmap owner] */
   BYTE CursorID;      /* Preferred cursor image ID */
};

struct RenderInfo { /* Version 2 */
   OBJECTID ParentID;   /* Object that contains the render area */
   OBJECTID BitmapID;   /* Drawable bitmap buffer */
   OBJECTID DataMID;    /* Bitmap data memory ID */
   OBJECTID FocusID;    /* Focus routine (equivalent to the RouteFocus field) */
   OBJECTID ScreenID;   /* If the render object is root, its screen is reflected here */
   LONG Flags;          /* Render flags (RNF_VISIBLE etc) */
   LONG XCoord;         /* Horizontal coordianate */
   LONG YCoord;         /* Vertical coordinate */
   LONG Width;          /* Width */
   LONG Height;         /* Height */
   union {
      LONG AbsX;        /* Absolute X */
      LONG Left;
   };
   union {
      LONG AbsY;        /* Absolute Y */
      LONG Top;
   };
   WORD Level;          /* Level within the hierarchy */
   BYTE BitsPerPixel;   /* Bits per pixel of the bitmap */
   BYTE BytesPerPixel;  /* Bytes per pixel of the bitmap */
   WORD TopLimit;       /* Top limit */
   WORD LeftLimit;      /* Left limit */
   WORD RightLimit;     /* Right limit */
   WORD BottomLimit;    /* Bottom limit */
   LONG LineWidth;      /* Line width of the bitmap, in bytes */
};

struct RenderInfoV1 {   /* Version 1 */
   OBJECTID ParentID;   /* Object that contains the render area */
   OBJECTID BitmapID;   /* Drawable bitmap buffer */
   OBJECTID DataMID;    /* Bitmap data memory ID */
   OBJECTID FocusID;    /* Focus routine (equivalent to the RouteFocus field) */
   OBJECTID ScreenID;   /* If the render object is root, its screen is reflected here */
   LONG Flags;          /* Render flags (RNF_VISIBLE etc) */
   WORD XCoord;         /* Horizontal coordianate */
   WORD YCoord;         /* Vertical coordinate */
   WORD Width;          /* Width */
   WORD Height;         /* Height */
   WORD AbsX;           /* Absolute X */
   WORD AbsY;           /* Absolute Y */
   WORD Level;          /* Level within the hierarchy */
   BYTE BitsPerPixel;   /* Bits per pixel of the bitmap */
   BYTE BytesPerPixel;  /* Bytes per pixel of the bitmap */
   WORD TopLimit;       /* Top limit */
   WORD LeftLimit;      /* Left limit */
   WORD RightLimit;     /* Right limit */
   WORD BottomLimit;    /* Bottom limit */
   LONG LineWidth;      /* Line width of the bitmap, in bytes */
};

/*** LockDrawableBitmap() return flags ***/

#define LVF_EXPOSECHANGES 0x00000001

/*** BlitDrawable() flags ***/

#define BDF_SYNC  0x00000001
#define BDF_SOLID 0x00000002

/*** AccessRenderList() flags ***/

#define ARF_READ    0x00000001  /* Read only access required */
#define ARF_WRITE   0x00000002  /* Full write access required */
#define ARF_UPDATE  0x00000004  /* Update a single entry (limited write access) */
#define ARF_NODELAY 0x00000008  /* Do not delay when waiting for access */

/*****************************************************************************
** Function Table.
*/

#ifndef LOCAL_RENDER

struct RenderBase {
   ERROR (*GetRenderInfo)(OBJECTID, struct RenderInfo *, LONG);
   ERROR (*LockDrawableBitmap)(OBJECTID, struct Bitmap **, LONG *);
   ERROR (*UnlockDrawableBitmap)(OBJECTID, struct Bitmap *);
   ERROR (*ExposeDrawable)(OBJECTID, LONG, LONG, LONG, LONG, LONG);
   ERROR (*BlitDrawable)(OBJECTID, struct Bitmap *, LONG, LONG, LONG, LONG, LONG, LONG, LONG);
   ERROR (*AccessDrawableSurface)(OBJECTID, struct BitmapSurface *, LONG);
   ERROR (*ReleaseDrawableSurface)(OBJECTID, struct BitmapSurface *);
   struct RenderList * (*AccessRenderList)(LONG, LONG *);
   void  (*ReleaseRenderList)(LONG);
   ERROR (*SetDrawableRegion)(OBJECTID, OBJECTPTR);
   ERROR (*ClippedCallback)(APTR, LONG, LONG, LONG, LONG);
   OBJECTID (*GetUserFocus)(void);
   void (*ForbidExpose)(void);
   void (*PermitExpose)(void);
};

#define AccessDrawableSurface(a,b,c)    (RenderBase->AccessDrawableSurface(a,b,c))
#define AccessRenderList(a,b)           (RenderBase->AccessRenderList(a,b))
#define BlitDrawable(a,b,c,d,e,f,g,h,i) (RenderBase->BlitDrawable(a,b,c,d,e,f,g,h,i))
#define ClippedCallback(a,b,c,d,e)      (RenderBase->ClippedCallback(a,b,c,d,e))
#define ExposeDrawable(a,b,c,d,e,f)     (RenderBase->ExposeDrawable(a,b,c,d,e,f))
#define ForbidExpose()                  (RenderBase->ForbidExpose())
#define GetRenderInfo(a,b,c)            (RenderBase->GetRenderInfo(a,b,c))
#define GetUserFocus()                  (RenderBase->GetUserFocus())
#define LockDrawableBitmap(a,b,c)       (RenderBase->LockDrawableBitmap(a,b,c))
#define PermitExpose()                  (RenderBase->PermitExpose())
#define ReleaseRenderList(a)            (RenderBase->ReleaseRenderList(a))
#define ReleaseDrawableSurface(a,b)     (RenderBase->ReleaseDrawableSurface(a,b))
#define SetDrawableRegion(a,b)          (RenderBase->SetDrawableRegion(a,b))
#define UnlockDrawableBitmap(a,b)       (RenderBase->UnlockDrawableBitmap(a,b))

#else

static ERROR AccessDrawableSurface(OBJECTID, struct BitmapSurface *, LONG);
static struct RenderList * AccessRenderList(LONG, LONG *);
static ERROR BlitDrawable(OBJECTID, struct Bitmap *, LONG, LONG, LONG, LONG, LONG, LONG, LONG);
static ERROR ClippedCallback(struct Render *, LONG, LONG, LONG, LONG);
static ERROR ExposeDrawable(OBJECTID, LONG, LONG, LONG, LONG, LONG);
static void ForbidExpose(void);
static ERROR GetRenderInfo(OBJECTID, struct RenderInfo *, LONG);
static OBJECTID GetUserFocus(void);
static ERROR LockDrawableBitmap(OBJECTID, struct Bitmap **, LONG *);
static void PermitExpose(void);
static ERROR ReleaseDrawableSurface(OBJECTID, struct BitmapSurface *);
static void  ReleaseRenderList(LONG);
static ERROR SetDrawableRegion(OBJECTID, OBJECTPTR);
static ERROR UnlockDrawableBitmap(OBJECTID, struct Bitmap *);

#endif /* LOCAL_RENDER */

#endif /* MODULES_RENDER_H */
